/*global
  console:false
  XShader:false
  clone:false
*/

function Multishader(src, gl, extensions, urls) {
    "use strict";
    this.parameters =
	{
            "camera_position" : [ 0.0, 0.0, 100.0],


            //"light_direction" : [1.0, 0.0, 0.0],
            "light_direction" : [0.7, 0.0, 0.7],



            "ambient" : [0.03, 0.03, 0.03],
            "diffuse_front" : [1.0, 1.0, 1.0],
            "diffuse_back" : [1.0, 1.0, 1.0],
            "shininess" : 0.0,
            "specular" : [0.0, 0.0, 0.0],

            "pattern_width" : 0.9,
            "pattern_exponent" : [4.0, 4.0],
            //"pattern_frequency" : [10.0, 10.0],
            "pattern_front_color" : [0.6, 0.6, 0.6],
            "pattern_back_color" : [0.6, 0.6, 0.6],

            "lattice_width" : 0.8,
            //"lattice_frequency" : [40.0, 40.0],

            "stripe_width" : 2,
            //"stripe_frequency" : [40.0, 40.0],

            "clipplane" : [0.0, 0.0, -1.0, 0.0],

            "pline_position" : [0.0, 0.0],
            "pline_width" : [0.06, 0.06],

            "pline_x_color" : [100.0/255, 0.0/255, 0.0/255],
            "pline_y_color" : [100.0/255, 0.0/255, 0.0/255],


            "curvature_color0" : [198.0/255.0, 99.0/255.0, 122.0/255.0],
            "curvature_color1" : [0.8, 0.6, 0.4]
	};

    this.gl = gl;
    this.src = src;
    this.urls = urls;

    var mode = {
	// 0
	// FIX
	//"stereographic" : surfaceMetadata[viewer.initial_surface].spaceform == 'S3',
	"stereographic" : false,
	// 1
	"specular" : true,
	// 2
	"clip" : false,
	// 3
	"pline" : 0,
	// 4
	"color" : 0,
	// 5
	"pattern" : 0,
	// 6
	"cutaway" : false,
	// 7
	"texclip" : 0,
	// 8
	"have_OES_standard_derivatives" : (extensions[0] !== null)
    };
    this._update(mode);
}

Multishader.prototype._update = function(mode) {
    "use strict";
    if (mode === undefined) {
	throw "multishader: mode is undefined";
    }
    if (this.mode == mode) {
	return;
    }

    // destroy shaders
    for (var i in this._currenShader) {
	this._currentShader[i].destroy();
    }
    this.mode = mode;

    this._currentShader = [
	new XShader(this.gl, this.mode, this.parameters, this.src, this.urls)
    ];

    // special 2-pass shader for parameter lines
    if (this.mode.pline !== 0)
    {
	var m = clone(this.mode);
	m.pline = 2;
	this._currentShader.push(
            new XShader(this.gl, m, this.parameters, this.src, this.urls) );
    }
};

Multishader.prototype._sync = function() {
    "use strict";
    for (var i in this._currentShader) {
	this._currentShader[i].sync();
    }
};

Multishader.prototype.subshader = function() {
    "use strict";
    return this._currentShader;
};

/*
  ----------------------------------------------------------------------
  set mode
  ----------------------------------------------------------------------
*/
Multishader.prototype.setStereographicProjection = function(state) {
    "use strict";
    var mode = clone(this.mode);
    mode.stereographic = state;
    this._update(mode);
};


Multishader.prototype.set_specular = function(state) {
    "use strict";
    var mode = clone(this.mode);
    mode.specular = state;
    this._update(mode);
};
Multishader.prototype.get_specular = function() {
    "use strict";
    return this.mode.specular;
};
Multishader.prototype.toggle_specular = function() {
    "use strict";
    this.set_specular( ! this.get_specular() );
};



Multishader.prototype.set_clipplane = function(state) {
    "use strict";
    var mode = clone(this.mode);
    mode.clip = state;
    this._update(mode);
};
Multishader.prototype.get_clipplane = function() {
    "use strict";
    return this.mode.clip;
};
Multishader.prototype.toggle_clipplane = function() {
    this.set_clipplane( ! this.get_clipplane() );
};


Multishader.prototype.set_pline = function(state) {
    "use strict";
    var mode = clone(this.mode);
    mode.pline = state ? 1 : 0;
    this._update(mode);
};
Multishader.prototype.get_pline = function() {
    "use strict";
    return this.mode.pline !== 0;
};
Multishader.prototype.toggle_pline = function() {
    "use strict";
    this.set_pline( ! this.get_pline() );
};



Multishader.prototype.getColorMode = function() {
    "use strict";
    return this.mode.color;
};
Multishader.prototype.setColorMode = function(state) {
    "use strict";
    var mode = clone(this.mode);
    mode.color = state;
    this._update(mode);
};

Multishader.prototype.get_pattern = function() {
    "use strict";
    return this.mode.pattern;
};
Multishader.prototype.set_pattern = function(state) {
    "use strict";
    var mode = clone(this.mode);
    mode.pattern = state;
    this._update(mode);
};

Multishader.prototype.get_texclip = function() {
    "use strict";
    return this.mode.texclip;
};
Multishader.prototype.set_texclip = function(state) {
    "use strict";
    var mode = clone(this.mode);
    mode.texclip = state;
    this._update(mode);
};

Multishader.prototype.set_cutaway = function(state) {
    "use strict";
    var mode = clone(this.mode);
    mode.cutaway = state;
    this._update(mode);
};
Multishader.prototype.get_cutaway = function() {
    "use strict";
    return this.mode.cutaway;
};
Multishader.prototype.toggle_cutaway = function() {
    "use strict";
    this.set_cutaway( ! this.get_cutaway() );
};



/*
  ----------------------------------------------------------------------
  set parameters
  ----------------------------------------------------------------------
*/
Multishader.prototype.setParameters = function(parameters) {
    "use strict";
    if (parameters.camera_position === undefined) {
	throw "[4]parameters.camera_position == undefined";
    }
    this.parameters.camera_position = parameters.camera_position;
    //this.parameters.pattern_frequency = parameters.pattern_frequency;
    //this.parameters.lattice_frequency = parameters.pattern_frequency;
    //this.parameters.stripe_frequency = parameters.stripe_frequency;
    this.parameters.clipplane = parameters.clipplane;
    this.parameters.pline_position = parameters.pline_position;
    if (this.parameters.camera_position === undefined) {
	throw "[3]parameters.camera_position == undefined";
    }
    this._sync();
};

Multishader.prototype.set_colors = function(color) {
    "use strict";
    this.parameters.ambient = color[0];
    this.parameters.diffuse_front = color[1];
    this.parameters.diffuse_back = color[2];
    this.parameters.pattern_front_color = color[3];
    this.parameters.pattern_back_color = color[4];
    this.parameters.pline_x_color = color[5];
    this.parameters.pline_y_color = color[6];
    this._sync();
};

Multishader.prototype.set_light_data = function(light_data) {
    "use strict";
    this.parameters.light_direction = light_data[0];
    this.parameters.specular = light_data[1];
    this.parameters.shininess = light_data[2];
    this._sync();
};
