/*global
  window:false
  document:false
  console:false
  disableSelection:false
  CONFIG:false
  Image:false
  Infobox:false
  surface_data:false
  baseURL:false
  Button:true
  ButtonBox:true
  _set_attributes:false
  _disabled:false
*/

var BUTTONBOX = {
    'surface' : 0,
    'shader' : 1
};
var BUTTON = {
    'info' : 0,
    'color' : 1,
    'pattern' : 2,
    'clipplane' : 3,
    'pline' : 4,
    'reload' : 5
};

//=========================================================
// TouchControls
//=========================================================

TouchControls = function(viewer) {
    "use strict";

/*
    this.background_color = [
        // no hover: [non-selected, selected]
        ['rgba(0, 0, 0, 0.0)',
         'rgba(169, 144, 163, 0.8)'
        ],
        // hover [non-selected, selected]
        ['rgba(255, 200, 0, 0.5)',
         'rgba(255, 200, 0, 0.5)']
    ];
    this.border_color = "rgba(60,60,60,1.0)";
*/
    this.background_color = [
        // no hover: [non-selected, selected]
        ['rgba(0, 0, 0, 0.0)', CONFIG.background_color ],
        // hover [non-selected, selected]
        ['rgba(255, 200, 0, 0.5)', 'rgba(255, 200, 0, 0.5)']
    ];
    this.border_color = CONFIG.border_color;


    this.viewer = viewer;
    this.menu = [];
    this.surface_item = [];
    
    this.color_item = [];
    this.cutaway_item = [];
    this.option_item = [];

    //this.panel_width = 180;
    this.widget_width = 120;
    this.widget_height = 22;

    this.viewer = viewer;


    this._init();
};

TouchControls.prototype._init = function()
{
    "use strict";
    this.buttonbox = [];

    this.buttonbox[BUTTONBOX.surface] = this.init_controls0();
    this.buttonbox[BUTTONBOX.shader] = this.init_controls1();
    this.infobox = new Infobox(this);

    if ( CONFIG.fps_counter ) {
        this.fps = this.create_fps_counter();
    }
    if ( CONFIG.triangle_counter ) {    
        this.triangle_count = this.create_triangle_counter();
    }

    if (CONFIG.mouse) {
        this.create_mouse_marker();
    }
};

//-------------------------------------------------
// surface button box
//-------------------------------------------------
TouchControls.prototype.init_controls0 = function()
{
    "use strict";
    var window_height = document.body.clientHeight;
    var window_width = document.body.clientWidth;
    var count = 6;
    var size = window_height / count;
    var geometry = [0, 0, size, size*count];


    function capture(touchcontrols0,callback0, i0) {
        return function(event) {
            return callback0(touchcontrols0, event, i0);
        };
    }

    var f = function(this0, event, i) {
        event.stopPropagation();
        event.preventDefault();
        var box = this0.buttonbox[BUTTONBOX.surface];

        // turn off all buttons
        for (var k in box.button)
        {
            box.button[k].set(false, false);
        }
        // turn on current button
        box.button[i].set(true, false);

        this0.infobox.set(i);
        this0.viewer.set_surface(i);

        return false;
    };


    var data = [];
    for (var i = 0; i < surface_data.length; ++i) {
        var s = surface_data[i];
        data[i] = {
            'label' : s.label,
            'url' : [baseURL + s.icon],
            'padding' : s.icon_padding,
            'callback' : capture(this, f, i)
        };
    }

    var x = new ButtonBox(
        this, document.body,
        data,
        geometry,
        this.background_color,
        this.border_color );

    x.viewer = this.viewer;

    // set the first button active
    x.button[0].set(true, false);

    return x;
};


function _set_color(viewer, i) {
    "use strict";
    viewer.color.set(i);
}

function _set_pattern(viewer, i) {
    "use strict";
    var choice = [
        [0, false],
        [3, true],
        [2, false],
        [5, true]
    ];
    viewer.multishader.set_pattern(choice[i][0]);
    viewer.multishader.set_cutaway(choice[i][1]);
}

function _set_clipplane(viewer, i) {
    "use strict";
    viewer.multishader.set_clipplane(i ? true :false);
}

function _set_pline(viewer, i) {
    "use strict";
    viewer.multishader.set_pline(i ? true :false);
}

function _reload() {
    "use strict";
    window.location.reload();
}

//-------------------------------------------------
// shader button box
//-------------------------------------------------
TouchControls.prototype.init_controls1 = function(callbacks)
{
    "use strict";
    var window_height = document.body.clientHeight;
    var window_width = document.body.clientWidth;
    var count = 6;
    var size = window_height / count;
    var geometry = [0, 0, size, size*count];

    function capture(touchcontrols0,callback0) {
        return function(event) {
            return callback0(touchcontrols0, event);
        };
    }

    function on_toggle_infobox(this0, event) {
        event.stopPropagation();
        event.preventDefault();
        this0.infobox.toggle();
        var box = this0.buttonbox[BUTTONBOX.shader];
        box.button[BUTTON.info].increment();
        return false;
    }
    function on_next_color(this0, event) {
        event.stopPropagation();
        event.preventDefault();
        var box = this0.buttonbox[BUTTONBOX.shader];
        var button = box.button[BUTTON.color];
        button.increment();
        _set_color(this0.viewer, button.counter);
        return false;
    }
    function on_next_pattern(this0, event) {
        event.stopPropagation();
        event.preventDefault();
        var box = this0.buttonbox[BUTTONBOX.shader];
        var button = box.button[BUTTON.pattern];
        button.increment();
        _set_pattern(this0.viewer, button.counter);
        return false;
    }
    function on_toggle_clipplane(this0, event) {
        event.stopPropagation();
        event.preventDefault();
        var box = this0.buttonbox[BUTTONBOX.shader];
        var button = box.button[BUTTON.clipplane];
        button.increment();
        _set_clipplane(this0.viewer, button.counter);
        return false;
    }
    function on_toggle_pline(this0, event) {
        event.stopPropagation();
        event.preventDefault();
        var box = this0.buttonbox[BUTTONBOX.shader];
        var button = box.button[BUTTON.pline];
        button.increment();
        _set_pline(this0.viewer, button.counter);
        return false;
    }
    function on_reload(this0, event) {
        event.stopPropagation();
        event.preventDefault();
        var box = this0.buttonbox[BUTTONBOX.shader];
        box.button[BUTTON.reload].set( true, false);
        _reload();
        return false;
    }

    geometry = [window_width-size, 0, size, size*count];

    var data = [
        // info
        {
            'label':'info',
            'url': [
                baseURL+'image/shader_buttons/info-small.png',
                baseURL+'image/shader_buttons/info-small.png' ],
            'callback': capture(this, on_toggle_infobox),
            "padding":10
        },
        // color
        {
            'label':'color',
            'url' : [
                baseURL+'image/shader_buttons/color0-small.png',
                baseURL+'image/shader_buttons/color1-small.png',
                baseURL+'image/shader_buttons/color2-small.png',
                baseURL+'image/shader_buttons/color3-small.png',
                baseURL+'image/shader_buttons/color4-small.png' ],
            'callback': capture(this, on_next_color),
            "padding":0
        },
        // pattern
        {
            'label':'pattern',
            'url': [
                baseURL+'image/shader_buttons/grid0-small.png',
                baseURL+'image/shader_buttons/grid1-trans-small.png',
                baseURL+'image/shader_buttons/grid2-small.png',
                baseURL+'image/shader_buttons/grid3-trans-small.png'],
            'callback': capture(this, on_next_pattern),
            "padding":0
        },
        // clipplane
        {
            'label':'clipplane',
            'url': [
                baseURL+'image/shader_buttons/clipplane-off-small.png',
                baseURL+'image/shader_buttons/clipplane-on-small.png' ],
            'callback': capture(this, on_toggle_clipplane),
            "padding":0
        },
        // pline
        {
            'label':'pline',
            'url': [
                baseURL+'image/shader_buttons/pline-off-crop-small.png',
                baseURL+'image/shader_buttons/pline-on-crop-small.png' ],
            'callback': capture(this, on_toggle_pline),
            "padding":0
        },
        // reload
        {
            'label':'reload',
            'url': [
                baseURL+'image/shader_buttons/reload-small.png'],
            'callback': capture(this, on_reload),
            "padding":10
        }
    ];

    return new ButtonBox(
        this,
        document.body,
        data,
        geometry,
        this.background_color,
        this.border_color );
};


TouchControls.prototype.create_fps_counter = function()
{
    "use strict";
    // fps counter
    var x = this.create_box(340);
    x.style.right = '340px';
    return x;
};

TouchControls.prototype.create_triangle_counter = function()
{
    "use strict";
    // triangle count
    var x = this.create_box(220);
    x.style.right = '220px';
    return x;
};

TouchControls.prototype.create_mouse_marker = function()
{
    "use strict";
    var x = this.create_box(100);
    x.innerHTML = "mouse enabled";
    return x;
};


TouchControls.prototype.create_box = function(xpos)
{
    "use strict";
    //--------------------------------------------
    var x = document.createElement('div');
    document.body.appendChild(x);
    _set_attributes(x);

    x.style.position = 'absolute';
    x.style.fontSize = '9pt';
    x.style.fontFamily = 'monospace';
    x.style.color = 'rgba(255,255,255,1.0)';
    x.style.backgroundColor = "rgba(100, 100, 100, 0.5)";

    x.style.position = 'absolute';
    x.style.bottom = '0px';
    x.style.right = xpos + 'px';
    x.style.width = this.widget_width + 'px';
    x.style.height = this.widget_height + 'px';

    x.style.lineHeight = this.widget_height + 'px';
    x.style.padding = "0px 0px 0px 0px";
    x.style.margin = "0px 0px 0px 0px";

    return x;
};

TouchControls.prototype.on_resize = function(event) {
    "use strict";

    var window_height = document.body.clientHeight;
    var window_width = document.body.clientWidth;
    var count = 6;
    var size = window_height / count;

    this.buttonbox[0].set_geometry(0);
    this.buttonbox[1].set_geometry(window_width-size);
};
