/*global
  Float32Array:false
*/


var xmat3 = xmat3 || {};


xmat3.identity = function(x) {
    "use strict";
    x[0] = 1.0;  x[3] = 0.0;  x[6] = 0.0;
    x[1] = 0.0;  x[4] = 1.0;  x[7] = 0.0;
    x[2] = 0.0;  x[5] = 0.0;  x[8] = 1.0;
};

/**
   Assigns b := a.
*/
xmat3.copy = function(b, a) {
    "use strict";
    b[0] = a[0]; b[3] = a[3]; b[6] = a[6];
    b[1] = a[1]; b[4] = a[4]; b[7] = a[7];
    b[2] = a[2]; b[5] = a[5]; b[8] = a[8];
};

xmat3.create = function(a) {
    "use strict";
    var x = new Float32Array(9);
    if (a)
    {
	xmat3.copy(x, a);
    }
    else
    {
	xmat3.identity(x);
    }
    return x;
};


//-------------------------------------------------------------------------------

var xmat4 = xmat4 || {};

xmat4.identity = function(x) {
    "use strict";
    x[ 0] = 1.0;  x[ 4] = 0.0;  x[ 8] = 0.0;  x[12] = 0.0;
    x[ 1] = 0.0;  x[ 5] = 1.0;  x[ 9] = 0.0;  x[13] = 0.0;
    x[ 2] = 0.0;  x[ 6] = 0.0;  x[10] = 1.0;  x[14] = 0.0;
    x[ 3] = 0.0;  x[ 7] = 0.0;  x[11] = 0.0;  x[15] = 1.0;
};

/**
   Assigns b := a.
*/
xmat4.copy = function(b, a) {
    "use strict";
    b[ 0] = a[ 0]; b[ 4] = a[ 4]; b[ 8] = a[ 8]; b[12] = a[12];
    b[ 1] = a[ 1]; b[ 5] = a[ 5]; b[ 9] = a[ 9]; b[13] = a[13];
    b[ 2] = a[ 2]; b[ 6] = a[ 6]; b[10] = a[10]; b[14] = a[14];
    b[ 3] = a[ 3]; b[ 7] = a[ 7]; b[11] = a[11]; b[15] = a[15];
};

xmat4.create = function(a) {
    "use strict";
    var x = new Float32Array(16);
    if (a)
    {
	xmat4.copy(x, a);
    }
    else
    {
	xmat4.identity(x);
    }
    return x;
};

/*
  Computes c = a * b.
  Allows a, b, c to overlap.
*/
xmat4.multiply = function(c, a, b)
{
    "use strict";
    var a00 = a[ 0], a01 = a[ 4], a02 = a[ 8], a03 = a[12],
    a10 = a[ 1], a11 = a[ 5], a12 = a[ 9], a13 = a[13],
    a20 = a[ 2], a21 = a[ 6], a22 = a[10], a23 = a[14],
    a30 = a[ 3], a31 = a[ 7], a32 = a[11], a33 = a[15];

    var b0 = b[ 0], b1 = b[ 1], b2 = b[ 2], b3 = b[ 3];
    c[ 0] = a00*b0 + a01*b1 + a02*b2 + a03*b3;
    c[ 1] = a10*b0 + a11*b1 + a12*b2 + a13*b3;
    c[ 2] = a20*b0 + a21*b1 + a22*b2 + a23*b3;
    c[ 3] = a30*b0 + a31*b1 + a32*b2 + a33*b3;

    b0 = b[ 4]; b1 = b[ 5]; b2 = b[ 6]; b3 = b[ 7];
    c[ 4] = a00*b0 + a01*b1 + a02*b2 + a03*b3;
    c[ 5] = a10*b0 + a11*b1 + a12*b2 + a13*b3;
    c[ 6] = a20*b0 + a21*b1 + a22*b2 + a23*b3;
    c[ 7] = a30*b0 + a31*b1 + a32*b2 + a33*b3;

    b0 = b[ 8]; b1 = b[ 9]; b2 = b[10]; b3 = b[11];
    c[ 8] = a00*b0 + a01*b1 + a02*b2 + a03*b3;
    c[ 9] = a10*b0 + a11*b1 + a12*b2 + a13*b3;
    c[10] = a20*b0 + a21*b1 + a22*b2 + a23*b3;
    c[11] = a30*b0 + a31*b1 + a32*b2 + a33*b3;
    
    b0 = b[12]; b1 = b[13]; b2 = b[14]; b3 = b[15];
    c[12] = a00*b0 + a01*b1 + a02*b2 + a03*b3;
    c[13] = a10*b0 + a11*b1 + a12*b2 + a13*b3;
    c[14] = a20*b0 + a21*b1 + a22*b2 + a23*b3;
    c[15] = a30*b0 + a31*b1 + a32*b2 + a33*b3;
};

/**
   Sets b = a * t, where t is a translation matrix.
   Allows a and b to overlap.
*/
xmat4.translate = function(b, a, t)
{
    "use strict";
    var t0 = t[0], t1 = t[1], t2 = t[2];
    if (b == a)
    {
	// last column
	b[12] = b[ 0]*t0 + b[ 4]*t1 + b[ 8]*t2 + b[12];
	b[13] = b[ 1]*t0 + b[ 5]*t1 + b[ 9]*t2 + b[13];
	b[14] = b[ 2]*t0 + b[ 6]*t1 + b[10]*t2 + b[14];
	b[15] = b[ 3]*t0 + b[ 7]*t1 + b[11]*t2 + b[15];
    }
    else
    {
	var a00 = a[ 0], a01 = a[ 4], a02 = a[ 8], a03 = a[12],
        a10 = a[ 1], a11 = a[ 5], a12 = a[ 9], a13 = a[13],
        a20 = a[ 2], a21 = a[ 6], a22 = a[10], a23 = a[14],
        a30 = a[ 3], a31 = a[ 7], a32 = a[11], a33 = a[15];

	// copy left 4x3 block
	b[ 0] = a00;  b[ 4] = a01; b[ 8] = a02;
	b[ 1] = a10;  b[ 5] = a11; b[ 9] = a12;
	b[ 2] = a20;  b[ 6] = a21; b[10] = a22;
	b[ 3] = a30;  b[ 7] = a31; b[11] = a32;
	
	// last column
	b[12] = a00*t0 + a01*t1 + a02*t2 + a03;
	b[13] = a10*t0 + a11*t1 + a12*t2 + a13;
	b[14] = a20*t0 + a21*t1 + a22*t2 + a23;
	b[15] = a30*t0 + a31*t1 + a32*t2 + a33;
    }
};
