/*global
  console:false
  xmat3
  xmat4:false
  sign:false
*/

/*
  Surface constructor
*/
XLAB.Surface = function(metaData, bufferData, textureData, colorData, indexData, mode) {
    "use strict";

    // copy metadata
    for (var i in metaData) {
        this[i] = metaData[i];
    }

    this.s3 = this.spaceform == 'S3';
    this.dim = this.s3 ? 4 : 3;
    this.textureStride = 2;
    this.colorStride = 1;
    this.bufferData = bufferData;
    this.textureData = textureData;
    this.colorData = colorData;
    this.indexData = indexData;
    this.mode = mode;

    // transforms
    if (this.transform) {
        var T0 = [];
        for (var i0 = 0; i0 < this.transform.length; ++i0) {
            T0[i0] = xmat4.create( this.transform[i0] );
        }
        this.transform = T0;
    }


    // texture transforms for plines
    if (this.texture_transform) {
        var T1 = [];
        for (var i1 = 0; i1 < this.texture_transform.length; ++i1) {
            T1[i1] = xmat3.create( this.texture_transform[i1] );
        }
        this.texture_transform1 = T1;
    }
    else
    {
        if (this.transform)
        {
            var T2 = [];
            for (var i2 = 0; i2 < this.transform.length; ++i2) {
                T2[i2] = xmat3.create();
            }
            this.texture_transform1 = T2;
        }
        else
        {
            this.texture_transform1 = [ xmat3.create() ];
        }
    }

    // frequency
    var f0 = this.uvFrequency[0];
    var f1 = this.uvFrequency[1];
    this.texture_transform0 = xmat3.create([f0, 0, 0, 0, f1, 0, 0, 0, 1] );


    /*
    // instance transforms
    // unflatten
    // compute orientation
    if (this.transform) {
    var T = [];
    //this.orientation = [];
    for (var ix = 0; ix < this.transform.length/16; ++ix) {
    var a = this.transform.slice( ix*16, (ix+1)*16 );
    var b = xmat4.create( a );
    T[ix] = b;
    //this.orientation[ix] = sign( mat4.determinant(b) );
    }
    this.transform = T;
    }

    if (this.texture_transform) {
    var T = [];
    //this.orientation = [];
    for (var ix = 0; ix < this.texture_transform.length/9; ++ix) {
    var a = this.texture_transform.slice( ix*9, (ix+1)*9 );
    var b = mat3.create( a );
    T[ix] = b;
    }
    this.texture_transform = T;
    }
    */
};


/**
   Load a surface.
*/
XLAB.Surface.prototype.load = function(gl) {
    "use strict";
    // vertex and normal
    this.vertex_and_normal = gl.createBuffer();
    gl.bindBuffer(gl.ARRAY_BUFFER, this.vertex_and_normal);
    gl.bufferData(gl.ARRAY_BUFFER, this.bufferData, gl.STATIC_DRAW);

    // texture
    if ('textureData' in this) {
        this.texture = gl.createBuffer();
        gl.bindBuffer(gl.ARRAY_BUFFER, this.texture);
        gl.bufferData(gl.ARRAY_BUFFER, this.textureData, gl.STATIC_DRAW);
    }

    /*
    // disabled
    // color
    if ('colorData' in this) {
    this.color = gl.createBuffer();
    gl.bindBuffer(gl.ARRAY_BUFFER, this.color);
    gl.bufferData(gl.ARRAY_BUFFER, this.colorData, gl.STATIC_DRAW);
    }
    */

    // index
    this.index = gl.createBuffer();
    gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, this.index);
    gl.bufferData(gl.ELEMENT_ARRAY_BUFFER, this.indexData, gl.STATIC_DRAW);
};

XLAB.Surface.prototype.unload = function(gl) {
    "use strict";
    // vertex and normal
    if ('vertex_and_normal' in this) {
        gl.deleteBuffer(this.vertex_and_normal);
        delete this.vertex_and_normal;
    }

    // texture
    if ('texture' in this) {
        gl.deleteBuffer(this.texture);
        delete this.texture;
    }

    // color
    if ('color' in this) {
        gl.deleteBuffer(this.color);
        delete this.color;
    }
    // index
    if ('index' in this) {
        gl.deleteBuffer(this.index);
        delete this.index;
    }
};


XLAB.Surface.prototype.enableVertexAttribArrays = function(gl,attribute)
{
    "use strict";
    gl.enableVertexAttribArray(attribute.position);
    gl.enableVertexAttribArray(attribute.normal);
    if ('texture' in attribute && 'texture' in this) {
        gl.enableVertexAttribArray(attribute.texture);
    }
    if ('color' in attribute && 'color' in this) {
        gl.enableVertexAttribArray(attribute.color);
    }
};

XLAB.Surface.prototype.disableVertexAttribArrays = function(gl,attribute)
{
    "use strict";
    gl.disableVertexAttribArray(attribute.position);
    gl.disableVertexAttribArray(attribute.normal);
    if ('texture' in attribute && 'texture' in this) {
        gl.disableVertexAttribArray(attribute.texture);
    }
    if ('color' in attribute && 'color' in this) {
        gl.disableVertexAttribArray(attribute.color);
    }
};

XLAB.Surface.prototype.draw1 = function(gl, attribute) {
    "use strict";

    var byte_stride = 8*this.dim; // vertex + normal
    var byte_offset0 = 0; // vertex
    var byte_offset1 = 4*this.dim; // vertex

    this.enableVertexAttribArrays(gl, attribute);

    gl.bindBuffer(gl.ARRAY_BUFFER, this.vertex_and_normal);
    // position
    //gl.enableVertexAttribArray(attribute.position);
    gl.vertexAttribPointer(attribute.position, this.dim, gl.FLOAT, false, byte_stride, byte_offset0);
    // normal
    //gl.enableVertexAttribArray(attribute.normal);
    gl.vertexAttribPointer(attribute.normal, this.dim, gl.FLOAT, false, byte_stride, byte_offset1);

    if ('texture' in attribute && 'texture' in this) {
        gl.bindBuffer(gl.ARRAY_BUFFER, this.texture);
        gl.enableVertexAttribArray(attribute.texture);
        gl.vertexAttribPointer( attribute.texture, this.textureStride, gl.FLOAT, false, 0, 0);
    }
    if ('color' in attribute && 'color' in this) {
        gl.bindBuffer(gl.ARRAY_BUFFER, this.color);
        //gl.enableVertexAttribArray(attribute.color);
        gl.vertexAttribPointer( attribute.color, this.colorStride, gl.FLOAT, false, 0, 0);
    }

    gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, this.index);

    var count = this.indexData.byteLength / 2;
    gl.drawElements(this.mode, count, gl.UNSIGNED_SHORT, 0);

    this.disableVertexAttribArrays(gl, attribute);
};


XLAB.Surface.prototype.drawN = function(gl, multishader, M, parts) {
    "use strict";

    multishader.use();

    if (this.transform && parts)
    {
        var tmp = xmat4.create();
        for (var i in parts)
        {
            var p = parts[i];
            var o = this.orientation[p];
            gl.frontFace(o>0 ? gl.CCW : gl.CW);
            var T = this.transform[p];
            if (T === undefined) {
                throw "missing transform";
            }
            //mat4.multiply(M, T, tmp);
            xmat4.multiply(tmp, M, T);
            multishader.setModelviewTransform(tmp);
            multishader.set_texture_transform0(this.texture_transform0);
            multishader.set_texture_transform1(this.texture_transform1[p]);
            this.draw1(gl, multishader.attribute());
        }
        gl.frontFace(gl.CCW);
    }
    else
    {
        multishader.setModelviewTransform(M);
        multishader.set_texture_transform0(this.texture_transform0);
        if (this.texture_transform1)
        {
            multishader.set_texture_transform1(this.texture_transform1[0]);
        }
        this.draw1(gl, multishader.attribute());
    }

    gl.bindBuffer(gl.ARRAY_BUFFER, null);
    gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, null);
};


XLAB.Surface.prototype.draw = function(gl, multishader, M, partno) {
    "use strict";
    // partno is 0,1,2
    if (this.parts)
    {
        this.drawN(gl, multishader.subshader()[0], M, this.parts[partno]);
        if (multishader.subshader().length > 1)
        {
            this.drawN(gl, multishader.subshader()[1], M, this.remaining_parts[partno]);
        }
    }
    else
    {
        this.drawN(gl, multishader.subshader()[0], M, [0]);
    }
};
