/*global
  global:false
  Image:false
  console:false
*/

/*
  Load a single image file.
  callback(image)
*/
function load_image(url, callback) {
    "use strict";
    var image = new Image();
    image.onload = function() {
        callback(image);
    };
    image.src = url;
}

/*
  Load multiple image files.
  callback(result)
  result is an array of images.
*/
function load_images(urls, callback) {
    "use strict";

    var total = urls.length;
    var count = 0;
    var result = [];

    function partialCallback(res, i) {
        result[i] = res;
        count++;
        
        if (count == total) {
            // we are done: call the callback with all the results
            callback(result);
        }
    }

    function capture(callback0, i0) {
        return function(res) { callback0(res,i0); };
    }

    // load each image
    for (var i in urls) {
        load_image( urls[i], capture(partialCallback,i) );
    }

    /*
    // load each image
    for (var i in urls) {
    load_image(
    urls[i],
    function(callback0, i0)
    {
    return function(res) { callback0(res,i0); };
    }(partialCallback,i)
    );
    }
    */
}

//-------------------------------------------------------
// texture
//-------------------------------------------------------
function _on_texture(image, gl, callback) {
    "use strict";
    var texture = gl.createTexture();

    var target = gl.TEXTURE_2D;
    gl.bindTexture(target, texture);

    //gl.texParameteri(target, gl.TEXTURE_WRAP_S, gl.CLAMP_TO_EDGE);
    //gl.texParameteri(target, gl.TEXTURE_WRAP_T, gl.CLAMP_TO_EDGE);
    gl.texParameteri(target, gl.TEXTURE_MIN_FILTER, gl.LINEAR_MIPMAP_LINEAR);
    gl.texParameteri(target, gl.TEXTURE_MAG_FILTER, gl.LINEAR);
    
    gl.pixelStorei(gl.UNPACK_FLIP_Y_WEBGL, false);
    gl.texImage2D(target, 0, gl.RGBA, gl.RGBA, gl.UNSIGNED_BYTE, image);

    gl.generateMipmap(target);
    
    gl.bindTexture(target, null);

    // user callback
    callback(texture);
}

XLAB.load_texture = function( gl, url, callback ) {
    "use strict";
    load_image(
        url,
        function(callback0, gl0, callback1)
        {
            return function(image) { callback0(image, gl0, callback1); };
        }(_on_texture, gl, callback)
    );
};


//-------------------------------------------------------
// cubemap
//-------------------------------------------------------
function _on_cubemap(image, gl, callback) {
    "use strict";

    var texture = gl.createTexture();

    var target = gl.TEXTURE_CUBE_MAP;
    gl.bindTexture(target, texture);

    gl.texParameteri(target, gl.TEXTURE_WRAP_S, gl.CLAMP_TO_EDGE);
    gl.texParameteri(target, gl.TEXTURE_WRAP_T, gl.CLAMP_TO_EDGE);
    gl.texParameteri(target, gl.TEXTURE_MIN_FILTER, gl.LINEAR);
    gl.texParameteri(target, gl.TEXTURE_MAG_FILTER, gl.LINEAR);
    
    gl.pixelStorei(gl.UNPACK_FLIP_Y_WEBGL, false);
    for (var i = 0; i < 6; ++i)
    {
        var face = gl.TEXTURE_CUBE_MAP_POSITIVE_X + i;
        gl.texImage2D(face, 0, gl.RGBA, gl.RGBA, gl.UNSIGNED_BYTE, image[i]);
    }
    
    gl.bindTexture(target, null);

    // user callback
    callback(texture);
}

XLAB.load_cubemap = function( gl, urls, callback ) {
    "use strict";
    load_images(
        urls,
        function(callback0, gl0, callback1)
        {
            return function(image) { callback0(image, gl0, callback1); };
        }(_on_cubemap, gl, callback)
    );
};
