// VERTEX SHADER

#if defined(PATTERN_SQUARE) || defined(PATTERN_CIRCLE) || defined(PATTERN_SUPERELLIPSE) || defined(PATTERN_STRIPE0) || defined(PATTERN_STRIPE1)
#define PATTERN
#endif

#if defined(PLINE_FULL) || defined(PLINE_PART)
#define PLINE
#endif

#if defined(PATTERN) || defined(PLINE) || defined(TEXIMAGE) || defined(TEXCLIP_X) || defined(TEXCLIP_Y)
#define TEXTURE
#endif

//#if defined(SPECULAR) || defined(CLIP) || defined(CUBEMAP)
#if defined(SPECULAR) || defined(CUBEMAP)
#define POSITION
#endif


#ifndef STEREOGRAPHIC_PROJECTION

// euclidean
attribute vec4 position;
attribute vec3 normal;
#if defined(TEXTURE)
attribute vec2 texture;
#endif
#if defined(LOCALCOLOR)
attribute float color;
#endif

uniform mat4 modelview_transform;
uniform mat4 projective_transform;
#if defined(TEXTURE)
uniform mat3 texture_transform0;
#endif
#if defined(PLINE)
uniform mat3 texture_transform1;
#endif

#if defined(POSITION)
varying vec4 position_i;
#endif
varying vec3 normal_i;
#if defined(TEXTURE)
varying vec2 texture_i;
varying vec2 texture_it;
#endif
#if defined(PLINE)
varying vec2 texture_j;
#endif
#if defined(LOCALCOLOR)
varying float color_i;
#endif
#if defined(CLIP)
uniform vec4 clipplane;
varying float clip_distance;
#endif

// R3
void main(void) {
  // position
  vec4 p = modelview_transform * position;
#if defined(CLIP)
  // clipplane is multiplied by modelview_transform
  // so that parts work correctly
  clip_distance = dot(p, clipplane);
#endif
#if defined(POSITION)
  position_i = p;
#endif
  gl_Position = projective_transform * p;
#if defined(TEXTURE)
  // texture
  texture_i = texture;
  texture_it = (texture_transform0 * vec3(texture,1.0)).xy;
#endif
#if defined(PLINE)
  // texture
  texture_j = (texture_transform1 * vec3(texture,1.0)).xy;
#endif
#if defined(LOCALCOLOR)
  color_i = color;
#endif

  // normal
  //normal_i = mat3(modelview_transform) * normal;

  /*
  // conversion from mat4 to mat3 seems borked with some cards
  mat3 normal_transform = mat3(
    modelview_transform[0].xyz,
    modelview_transform[1].xyz,
    modelview_transform[2].xyz );
  */
  mat3 normal_transform = mat3(modelview_transform);

  normal_i = normal_transform * normal;
}

#else

// stereographic projection
attribute vec4 position;
attribute vec4 normal;
#if defined(TEXTURE)
attribute vec2 texture;
#endif
#if defined(LOCALCOLOR)
attribute float color;
#endif

uniform mat4 modelview_transform;
uniform mat4 projective_transform;
#if defined(TEXTURE)
uniform mat3 texture_transform0;
#endif
#if defined(PLINE)
uniform mat3 texture_transform1;
#endif

#if defined(POSITION)
varying vec4 position_i;
#endif
varying vec3 normal_i;
#if defined(TEXTURE)
varying vec2 texture_i;
varying vec2 texture_it;
#endif
#if defined(PLINE)
varying vec2 texture_j;
#endif
#if defined(LOCALCOLOR)
varying float color_i;
#endif
#if defined(CLIP)
uniform vec4 clipplane;
varying float clip_distance;
#endif

// S3
void main(void) {

  // compute the position under stereographic projection
  vec4 p = modelview_transform * position;
#if defined(CLIP)
  // clipplane is multiplied by modelview_transform
  // so that parts work correctly
  clip_distance = dot(p, clipplane);
#endif
  float d = 1.0 + p.w;
  p.w = d;
  // transform the position
  gl_Position = projective_transform * p;

#if defined(POSITION)
  position_i = p;
#endif
#if defined(TEXTURE)
  texture_i = texture;
  texture_it = (texture_transform0 * vec3(texture,1.0)).xy;
#endif
#if defined(PLINE)
  texture_j = (texture_transform1 * vec3(texture,1.0)).xy;
#endif
#if defined(LOCALCOLOR)
  color_i = color;
#endif

  // compute the normal under stereographic projection
  vec4 normal4 = modelview_transform * normal;
  // note wrong sign
  normal_i = -normalize( d * normal4.xyz - normal4.w * p.xyz );
}

#endif
